// SpamMailEraser
//   Command.c++
//       2004 Y.Senta

#include <stdlib.h>
#include <ctype.h>
#include "Command.h"

bool Command::CheckVariable(const char *c){
  char *p=(char *)c;
  while (*p){
    if (!isalpha(*p) && !isdigit(*c) && *c !='-' && *c != '_')
      return false;
    p++;
  }
  return true;
}

bool Command::IsHeader(const char *c){
  int n = strlen(c);
  if (n==0)
    return false;
  if (c[n-1] != ':')
    return false;
  return true;
}

bool Command::Load(const char *szFileName){
  FILE *fp = fopen(szFileName, "rb");
  if (!fp) {
    fprintf (stderr, "Error: Can not open [%s] file\n", szFileName);
    return false;
  }
  char buf[FILELINESIZE];
  bool bCmdMode = false;
  
  int l=0;
  CmdLine *pTop = NULL;
  while (fgets(buf, sizeof(buf), fp)){
    l++;
    if (buf[0]=='#')
      continue;
    Chop(buf);
    
    char *p=SkipSpace(buf);
    
    if (p[0]==0)
      continue;
    if (bCmdMode){
      CmdLine *pTmp = new CmdLine;
      
      if (!strncmp(p, "SET", 3)){
	p = SkipSpace(p+3);
	char *q=NextToken(p);
	if (!CheckVariable(p)){
	  fprintf (stderr, "Error (%d)\n", l);
	  return false;
	}
	pTmp ->m_eCmd = E_SET;
	pTmp->m_strValue = p;
	
	q=SkipSpace(q);
	p=NextToken(q);
	
	if (!IsHeader(q)){
	  fprintf (stderr, "Error (%d)\n", l);
	  return false;
	}
	
	pTmp->m_strPara1 = q;
	pTmp->m_strPara2 = SkipSpace(p);
      } else if (!strncmp(p, "LET", 3)){
	p = SkipSpace(p+3);
	char *q=NextToken(p);
	if (!CheckVariable(p)){
	  fprintf (stderr, "Error (%d)\n", l);
	  return false;
	}
	pTmp ->m_eCmd = E_LET;
	pTmp->m_strValue = p;
	pTmp->m_strPara1 = SkipSpace(q);
	
	if (pTmp->m_strPara1.IsEmpty()){
	  fprintf (stderr, "Error (%d)\n", l);
	  return false;
	}
      } else if (!strncmp(p, "REJECTIF", 8)) {
	p = SkipSpace(p+8);
	char *q=NextToken(p);
	if (!CheckVariable(p)){
	  fprintf (stderr, "Error (%d)\n", l);
	  return false;
	}
	pTmp ->m_eCmd = E_REJECT;
	pTmp->m_strValue = p;
      } else if (!strncmp(p, "ACCEPTIF", 8)) {
	p = SkipSpace(p+8);
	char *q=NextToken(p);
	if (!CheckVariable(p)){
	  fprintf (stderr, "Error (%d)\n", l);
	  return false;
	}
	pTmp ->m_eCmd = E_ACCEPT;
	pTmp->m_strValue = p;
      } else if (!strncmp(p, "PRINT", 5)) {
	p = SkipSpace(p+5);
	pTmp->m_eCmd = E_PRINT;
	pTmp->m_strPara1 = p;
      } else if (!strncmp(p, "DUMP", 4)) {
	pTmp->m_eCmd = E_DUMP;
      } else {
	fprintf (stderr, "Error (%d)\n", l);
	return false;
      }
      pTmp->m_pNext = pTop;
      pTop = pTmp;
    } else {
      if (!strncmp(p, "HOST", 4))
	m_strHost = SkipSpace(p+4);
      else if (!strncmp(p, "ID", 2))
	m_strID = SkipSpace(p+2);
      else if (!strncmp(p, "PASS", 4))
	m_strPass = SkipSpace(p+4);
      else if (!strncmp(p, "APOP", 4))
	m_bAPOPMode=true;
      else if (!strncmp(p, "POP3", 4))
	m_bAPOPMode=false;
      else if (!strncmp(p, "Command", 7))
	bCmdMode = true;
      else {
	fprintf (stderr, "Error (%d)\n", l);
	return false;
      }
    }
  }
  fclose(fp);
  
  while (pTop){
    CmdLine *pNext = pTop->m_pNext;
    pTop->m_pNext = m_pTop;
    m_pTop = pTop;
    pTop=pNext;
  }
  
  return true;
}

bool Command::DoCheck(MyStringList &lst){
  m_Variable.Clear();
  
  CmdLine *pTmp = m_pTop;
  bool bRet, bValue;
  while (pTmp){
    switch(pTmp->m_eCmd){
    case E_SET:
      {
	char *p=pTmp->m_strPara2.GetBuffer();
	if (p[0]=='\\'){
	  if (p[1]=='\\'){
	    p+=2;
	  } else {
	    if (!strncmp(p+1, "8bit", 4)){
	      bValue = Check8bitCode(pTmp->m_strPara1.GetBuffer(), lst);
	      m_Variable.Set(pTmp->m_strValue.GetBuffer(), bValue);
	      break;
	    } else if (!strncmp(p+1, "nocase", 6)){
	      p=SkipSpace(p+7);
	      bValue = FindString(pTmp->m_strPara1.GetBuffer(), p, lst, true);
	      m_Variable.Set(pTmp->m_strValue.GetBuffer(), bValue);
	      break;
	    } else if (!strncmp(p+1, "any", 3)){
	      char *pHeader = pTmp->m_strPara1.GetBuffer();
	      if (*pHeader=='$')
		pHeader++;
	      if (*pHeader=='_'){
		bValue=true;
	      } else {
		bValue=false;
		MyStringListItem *pTmp = lst.GetTop();
		int len = strlen(pHeader);
		while (pTmp){
		  char *p=pTmp->GetItem();
		  if(!strncasecmp(p, pHeader, len)){
		    bValue=true;
		    break;
		  }
		  pTmp=lst.GetNext(pTmp);
		}
	      }
	      m_Variable.Set(pTmp->m_strValue.GetBuffer(), bValue);
	      break;
	    }
	  
	    fprintf(stderr, "Error: %s is not command.\n", p+1);
	    exit(2);
	  }
	}
	bValue = FindString(pTmp->m_strPara1.GetBuffer(), p, lst, false);
	m_Variable.Set(pTmp->m_strValue.GetBuffer(), bValue);
      }
      break;
    case E_LET:
      bValue = Calc(pTmp->m_strPara1.GetBuffer());
      m_Variable.Set(pTmp->m_strValue.GetBuffer(), bValue);
      break;
    case E_ACCEPT:
    case E_REJECT:
      bRet = m_Variable.Get(&bValue, pTmp->m_strValue.GetBuffer());
      if (!bRet){
	fprintf(stderr, "Error: %s is not defined.\n", pTmp->m_strValue.GetBuffer());
	exit(2);
      }
      if (pTmp->m_eCmd == E_ACCEPT && bValue)
	return true;
      if (pTmp->m_eCmd == E_REJECT && bValue)
	return false;
      break;
    case E_PRINT:
      printf ("%s\n", pTmp->m_strPara1.GetBuffer());
      break;
    case E_DUMP:
      m_Variable.Dump(stdout);
      break;
    }
    pTmp = pTmp->m_pNext;
  }
  return true;
}

bool Command::FindString(const char *szHeader, const char *szPattern, MyStringList &lst, bool bNocase){
  MyStringListItem *pTmp = lst.GetTop();
  bool bCheck = false;
  int len = strlen(szHeader);
  int offset = 0;
  
  if (szHeader[0]=='$'){
    offset = 1;
    len--;
  }
  if (szHeader[offset]=='_')
    bCheck = true;

  while (pTmp){
    char *p=pTmp->GetItem();
    if (szHeader[offset]!='_'){
       if(!strncasecmp(p, szHeader+offset, len))
	bCheck = true;
      else
	bCheck = false;
    }
    if (bCheck){
      MyString str = p;
      MyStringListItem *pTmp2 = lst.GetNext(pTmp);
      while (pTmp2){
	p = pTmp2->GetItem();
	if (p[0]==' ' || p[0] == '\t'){
	  if (str.GetSize() + strlen(p) + 3 >= ONELINESIZE){
	    if (StrFind(str.GetBuffer(), szPattern, bNocase, offset ? true : false))
	      return true;
	    str = "";
	  } else {
	    str += "\r\n\t";
	  }
	  str += SkipSpace(p);
	  pTmp2 = lst.GetNext(pTmp2);
	} else {
	  break;
	}
      }
      if (StrFind(str.GetBuffer(), szPattern, bNocase, offset ? true : false))
	return true;
      pTmp = pTmp2;
    } else {
      pTmp=lst.GetNext(pTmp);
    }
  }
  
  return false;
}

bool Command::Check8bitCode(const char *szHeader, MyStringList &lst){
  MyStringListItem *pTmp = lst.GetTop();
  
  bool bCheck = false;
  int len = strlen(szHeader);
  int offset = 0;
  
  if (szHeader[0]=='$'){
    offset = 1;
    len--;
  }
  if (szHeader[offset]=='_')
    bCheck = true;
  
  while (pTmp){
    char *p=pTmp->GetItem();
    if (szHeader[offset]!='_' && p[0]!=' ' && p[0] != '\t'){
      if (!strncasecmp(p, szHeader+offset, len))
	bCheck = true;
      else
	bCheck = false;
    }
    if (bCheck){
      while (*p){
	if (*p & 0x80)
	  return true;
	p++;
      }
    }
    
    pTmp=lst.GetNext(pTmp);
  }
  
  return false;
}

bool Command::Calc(const char *szCmd){
  MyString str;
  MyString var;
  
  char *p=(char *)szCmd;
  bool bIn=false;
  bool bRet, bValue;
  
  while (*p){
    if (*p !=' ' && *p != '\t') {
      if (bIn){
	if (*p == '(' || *p == ')' || *p == '!' || *p == '&' || *p == '|'){
	  bIn = false;
	  bRet = m_Variable.Get(&bValue, var.GetBuffer());
	  if (!bRet){
	    fprintf(stderr, "Error: %s is not defined.\n", var.GetBuffer());
	    exit(2);
	  }
	  str += bValue ? '1' : '0';
	  str += *p;
	} else {
	  var += *p;
	}
      } else {
	if (*p != '(' && *p != ')' && *p != '!' && *p != '&' && *p != '|'){
	  bIn = true;
	  var = *p;
	} else {
	  str += *p;
	}
      }
    }
    p++;
  }
  if (bIn){
    bRet = m_Variable.Get(&bValue, var.GetBuffer());
    if (!bRet){
      fprintf(stderr, "Error: %s is not defined.\n", var.GetBuffer());
      exit(2);
    }
    str += bValue ? '1' : '0';
  }
  p=new char[str.GetSize()+1];
  
  strcpy(p, str.GetBuffer());
  if (!DoCalc(p)){
    fprintf(stderr, "Error: Can not calculatin [%s].\n", szCmd);
    exit(2);
  }

  bool r = p[0] == '1' ? true : false;
  delete [] p;
  return r;
}

void MemoryMove(char *p, int n){
  char *q;
  for (q = p+1;*q; q++){
    *q = *(q+n);
  }
  *q = 0;
}

bool Command::DoCalc(char *p){
  char *q;
  bool bCalc;
  do {
    bCalc = true;
    
    q=strstr(p, "!0");
    if (q){
      *q='1';
      MemoryMove(q, 1);
      continue;
    }
    q=strstr(p, "!1");
    if (q){
      *q='0';
      MemoryMove(q, 1);
      continue;
    }
    
    bCalc = false;
    q=strstr(p, "0&0");
    if (q){
      *q='0';
      MemoryMove(q, 2);
      bCalc = true;
    }
    q=strstr(p, "0&1");
    if (q){
      *q='0';
      MemoryMove(q, 2);
      bCalc = true;
    }
    q=strstr(p, "1&0");
    if (q){
      *q='0';
      MemoryMove(q, 2);
      bCalc = true;
    }
    q=strstr(p, "1&1");
    if (q){
      *q='1';
      MemoryMove(q, 2);
      bCalc = true;
    }
    
    q=strstr(p, "0|0");
    if (q){
      *q='0';
      MemoryMove(q, 2);
      bCalc = true;
    }
    q=strstr(p, "0|1");
    if (q){
      *q='1';
      MemoryMove(q, 2);
      bCalc = true;
    }
    q=strstr(p, "1|0");
    if (q){
      *q='1';
      MemoryMove(q, 2);
      bCalc = true;
    }
    q=strstr(p, "1|1");
    if (q){
      *q='1';
      MemoryMove(q, 2);
      bCalc = true;
    }
    
    q=strstr(p, "(0)");
    if (q){
      *q='0';
      MemoryMove(q, 2);
      bCalc = true;
    }
    q=strstr(p, "(1)");
    if (q){
      *q='1';
      MemoryMove(q, 2);
      bCalc = true;
    }
  } while (bCalc);
  
  if ((p[0] == '0' || p[0]=='1') && p[1]==0)
    return true;
  
  return false;
}
