// SpamMailEraser
//   Command.h
//       2004 Y.Senta

#ifndef _COMMAND_H_
#define _COMMAND_H_

#include "CommonLib.h"

class Variable{
  friend class VariableList;
  
  MyString m_strName;
  bool m_bValue;
  
  Variable *m_pNext;
 public:
  Variable(){
    m_bValue = false;
    m_pNext = NULL;
  }
  Variable(const char *c, bool b){
    m_strName = c;
    m_bValue = b;
    
    m_pNext = NULL;
  }
  ~Variable(){
    if (m_pNext)
      delete m_pNext;
  }
  
  void SetValue(bool b){
    m_bValue = b;
  }
  bool GetValue(){
    return m_bValue;
  }
};

class VariableList{
  Variable *m_pTop;
 public:
  VariableList(){
    m_pTop = NULL;
  }
  ~VariableList(){
    if (m_pTop)
      delete m_pTop;
  }
  
  bool Set(const char *c, bool b){
    Variable *pTmp = m_pTop;
    while (pTmp){
      if (pTmp->m_strName == c){
	pTmp->SetValue(b);
	return true;
      }
      pTmp=pTmp->m_pNext;
    }
    pTmp=new Variable(c, b);
    pTmp->m_pNext=m_pTop;
    m_pTop=pTmp;
    
    return true;
  }
  bool Get(bool *b, const char *c){
    Variable *pTmp = m_pTop;
    while (pTmp){
      if (pTmp->m_strName == c){
	*b=pTmp->GetValue();
	return true;
      }
      pTmp=pTmp->m_pNext;
    }
    return false;
  }
  
  void Clear(){
    if (m_pTop)
      delete m_pTop;
    m_pTop = NULL;
  }
  
  void Dump(FILE *fp){
    Variable *pTmp = m_pTop;
    while (pTmp){
      fprintf(fp, "%s:%c\n", pTmp->m_strName.GetBuffer(), pTmp->m_bValue ? 'o' : 'x');
      pTmp=pTmp->m_pNext;
    }
    
  }
};

enum eCmd{
  E_SET,
  E_LET,
  E_ACCEPT,
  E_REJECT,
  E_PRINT,
  E_DUMP
};

class CmdLine {
  friend class Command;
  
  eCmd m_eCmd;
  MyString m_strValue;
  MyString m_strPara1;
  MyString m_strPara2;
  
  CmdLine *m_pNext;
 public:
  CmdLine(){
    m_pNext = NULL;
  }
  ~CmdLine(){
    if (m_pNext)
      delete m_pNext;
  }
};

class Command{
  MyString m_strHost;
  MyString m_strID;
  MyString m_strPass;
  bool m_bAPOPMode;
  
  CmdLine *m_pTop;
  
  VariableList m_Variable;
  
  bool CheckVariable(const char *);
  bool IsHeader(const char *);
  
  bool FindString(const char *, const char *, MyStringList &, bool);
  bool Check8bitCode(const char *, MyStringList &);
 public:
  Command(){
    m_pTop = NULL;
    m_bAPOPMode=false;
  }
  ~Command(){
    if (m_pTop)
      delete m_pTop;
  }
  
  bool Calc(const char *);
  bool DoCalc(char *);
  
  bool Load(const char *);
  bool DoCheck(MyStringList &);
  
  char *GetHost(){
    return m_strHost.GetBuffer();
  }
  char *GetID(){
    return m_strID.GetBuffer();
  }
  char *GetPass(){
    return m_strPass.GetBuffer();
  }
  bool IsAPOPMode(){
    return m_bAPOPMode;
  }
};

#endif
