// SpamMailEraser
//   CommonLib.c++
//       2004 Y.Senta

#include "CommonLib.h"
#include <stdio.h>

#ifdef HAVE_GNU_RX
extern "C" {
#ifndef __STDC__
#define __STDC__
#endif
#include <rxposix.h>
}
#endif

void Chop(char *c){
  for (int i=strlen(c)-1;i>=0;i--){
    if (c[i]=='\n' || c[i]=='\r')
      c[i]=0;
    else
      break;
  }
}

char *SkipSpace(char *c){
  char *p=c;
  
  while (*p){
    if (*p!=' ' && *p!='\t')
      break;
    p++;
  }
  return p;
}

char *NextToken(char *c){
  char *p=SkipSpace(c);
  char *q=p;
  while (*q){
    if (*q==' ' || *q=='\t'){
      *q=0;
      q++;
      break;
    }
    q++;
  }
  return q;
}

void JIS2EUC(const char* szJIS, char *szEUC){
  int i = 0, iR = 0;
  bool bSTART = false;
  
  int len = strlen(szJIS);
  
  for (;;) {
    if (szJIS[i]==0x1b){
      if (i+2 < len && 
	  szJIS[i + 1] == 0x24 && 
	  (szJIS[i + 2] == 0x42 || szJIS[i + 2] == 0x40)) {
	bSTART = true;
	i += 3;
	continue;
      }
      if (i+2 < len && 
	  szJIS[i+ 1] == 0x28 && 
	  (szJIS[i + 2] == 0x42 || szJIS[i + 2] == 0x4A)) {
	bSTART = false;
	i += 3;
	continue;
      }
      if (i+3 < len && 			
	  szJIS[i + 1] == 0x24 &&
	  szJIS[i + 2] == 0x28 &&
	  szJIS[i + 3] == 0x44) {
	bSTART = true;
	i += 4;
	continue;
      }
      if (i+5 < len &&
	  szJIS[i + 1] == 0x26 &&
	  szJIS[i + 2] == 0x40 &&
	  szJIS[i + 3] == 0x1B &&
	  szJIS[i + 4] == 0x24 &&
	  szJIS[i + 5] == 0x42) {
	bSTART = true;
	i += 6;
	continue;
      }
    }
    
    if (szJIS[i] >= 0x21 && szJIS[i] <= 0x7e && bSTART) {
      szEUC[iR] = szJIS[i] | 0x80;
      szEUC[iR + 1] = szJIS[i + 1] | 0x80;
      i += 2;
      iR += 2;
      continue;
    }
    if (szJIS[i] == '\0') {
      szEUC[iR] = szJIS[i];
      break;
    }
    szEUC[iR] = szJIS[i];
    iR++;
    i++;
  }
}

char charconv(char c) {
  if (c >= 'A' && c <= 'Z')
    return (c - 'A');
  if (c >= 'a' && c <= 'z')
    return (c - 'a' + 0x1a);
  if (c >= '0' && c <= '9')
    return (c - '0' + 0x34);
  if (c == '+')
    return 0x3e;
  if (c == '/')
    return 0x3f;
  if (c == '=')
    return '\0';
  return '\0';
}

void DecodeMIME(const char *szFrom, char *szTo) {
  int len, i, iR;
  char a1, a2, a3, a4;
  
  len = strlen(szFrom);
  i = 0;
  iR = 0;
  while (1) {
    if (i >= len)
      break;
    a1 = charconv(szFrom[i]);
    a2 = charconv(szFrom[i+1]);
    a3 = charconv(szFrom[i+2]);
    a4 = charconv(szFrom[i+3]);
    szTo[iR] = (a1 << 2) + (a2 >>4);
    szTo[iR + 1] = (a2 << 4) + (a3 >>2);
    szTo[iR + 2] = (a3 << 6) + a4;
    i += 4;
    iR += 3;
  }
  szTo[iR]=0;
  return;
}

void MIME2EUC(MyString &strRet, const char *szOrg){
  char *token;
  
  char szBuf[ONELINESIZE], szBuf2[ONELINESIZE];
  int len, mod;
  bool bCopy = true;
  
  token = strtok((char *)szOrg, "=\r\n\t");
  while (token != NULL) {
    if (strstr(token, "?ISO-2022-JP?B?") != token &&
	strstr(token, "?iso-2022-jp?B?") != token) {
      if (strcmp(token, "?") != 0 && bCopy) {
	JIS2EUC(token, szBuf);
	strRet += szBuf;
      }
    } else {
      strcpy(szBuf, token + 15);
      if (szBuf[strlen(szBuf) - 1] == '?')
	szBuf[strlen(szBuf) - 1] = '=';
      for (len = strlen(szBuf)-1; len >=0; len--){
	if (szBuf[len]!='=')
	  break;
	szBuf[len]=0;
      }
      len = strlen(szBuf);
      
      mod = len % 4;
      switch (mod) {
      case 1:
	strcat(szBuf, "===");
	bCopy = false;
	break;
      case 2:
	strcat(szBuf, "==");
	bCopy = false;
	break;
      case 3:
	strcat(szBuf, "=");
	bCopy = false;
	break;
      }
      DecodeMIME(szBuf, szBuf2);
      strRet += szBuf2;
    }
    bCopy = true;
    token = strtok(NULL, "=\r\n\t");
  }
  JIS2EUC(strRet.GetBuffer(), szBuf);
  strRet = szBuf;
}

bool MyStringList::LoadFile(const char *szFileName){
  FILE *fp=fopen(szFileName, "r");
  if (!fp)
    return false;
  
  char buf[FILELINESIZE];
  
  MyStringListItem *pTop=NULL;
  while(fgets(buf, sizeof(buf), fp)){
    Chop(buf);
    if (!buf[0])
      break;
    MyStringListItem *pTmp=new MyStringListItem(buf);
    pTmp->m_pNext = pTop;
    pTop = pTmp;
  }
  fclose(fp);
  
  while (pTop){
    MyStringListItem *pNext = pTop->m_pNext;
    pTop->m_pNext = m_pTop;
    m_pTop = pTop;
    pTop=pNext;
  }
  return true;
}

bool MyStringList::SaveFile(const char *szFileName){
  FILE *fp=fopen(szFileName, "w");
  if (!fp)
    return false;
  MyStringListItem *pTmp = m_pTop;
  while (pTmp){
    fprintf(fp, "%s\n", pTmp->m_Data.GetBuffer());
    pTmp=pTmp->m_pNext;
  }
  fclose(fp);
  return true;
}

bool StrFind(const char *szStr, const char *szPattern, bool bNocase, bool bMIME){
  char *p, *s;
  char bufS[ONELINESIZE], bufP[ONELINESIZE];
  if (bNocase){
    strcpy(bufS, szStr);
    strcpy(bufP, szPattern);

    int i;
    for (i=0;bufS[i];i++){
      if (bufS[i] >= 'A' && bufS[i] <='Z')
	bufS[i] += 'a' - 'A';
    }
    for (i=0;bufP[i];i++){
      if (bufP[i] >= 'A' && bufP[i] <='Z')
	bufP[i] += 'a' - 'A';
    }
    p=bufP;
    s=bufS;
  } else {
    p=(char *)szPattern;
    s=(char *)szStr;
  }

  if (bMIME) {
    MyString strMIME;
    MIME2EUC(strMIME, szStr);
    s = strMIME.GetBuffer();
    return strstr(s, p) ? true : false;
  } 
  
#ifdef HAVE_GNU_RX
  regex_t preg;
  regmatch_t pmatch[4];
  
  regcomp(&preg, p, REG_EXTENDED|REG_NEWLINE);
  regexec(&preg, s, 4, pmatch, 0);
  regfree(&preg);
  if (pmatch[0].rm_eo-pmatch[0].rm_so)
    return true;
  else
    return false;
#else
  return strstr(s, p) ? true : false;
#endif
}
