// SpamMailEraser
//   CommonLib.h
//       2004 Y.Senta

#ifndef _COMMONLIB_H_
#define _COMMONLIB_H

#include <stdio.h>
#include <string.h>

#define ONELINESIZE 1024 // LINESIZE > 1000 (RFC 2822)
#define FILELINESIZE ONELINESIZE

class MyString {
  char *m_p;
  int m_n;
 public:
  MyString(){
    m_p=new char [1];
    m_p[0]=0;
    m_n=0;
  }
  MyString(const char *c){
    m_n = strlen(c);
    m_p = new char [m_n+1];
    strcpy(m_p, c);
  }
  
  ~MyString(){
    delete [] m_p;
  }
  
  char *GetBuffer(){
    return m_p;
  }
  
  bool IsEmpty(){
    return m_n ? false : true;
  }
  int GetSize(){
    return m_n;
  }
  
  MyString & operator = (const char *c) {
    if (m_p)
      delete [] m_p;
    
    m_n = strlen(c);
    m_p = new char [m_n+1];
    strcpy(m_p, c);
    
    return *this;
  }
  MyString & operator = (const char c) {
    if (m_p)
      delete [] m_p;
    
    m_p = new char [2];
    m_p[0]=c;
    m_p[1]=0;
    m_n=1;
    
    return *this;
  }
  MyString & operator += (const char *c) {
    m_n += strlen(c);
    char *pTmp = new char [m_n+1];
    if (m_p){
      strcpy(pTmp, m_p);
      strcat(pTmp, c);
    } else {
      strcpy(pTmp, c);
    }
    if (m_p)
      delete [] m_p;
    m_p = pTmp;
    
    return *this;
  }
  
  MyString & operator += (const char c) {
    m_n += 1;
    char *pTmp = new char [m_n+1];
    if (m_p){
      strcpy(pTmp, m_p);
      pTmp[m_n-1]=c;
    } else {
      pTmp[0]=c;
    }
    pTmp[m_n]=0;
    if (m_p)
      delete [] m_p;
    m_p = pTmp;
    
    return *this;
  }
  
  friend bool operator == (MyString &s, const char *c){
    return strcmp(s.GetBuffer(), c) ? false : true;
  }
};

class MyStringListItem{
  friend class MyStringList;
  
  MyString m_Data;
  MyStringListItem *m_pNext;
 public:
  MyStringListItem(){
    m_pNext=NULL;
  }
  MyStringListItem(const char *c){
    m_Data = c;
    m_pNext=NULL;
  }
  ~MyStringListItem(){
    if (m_pNext)
      delete m_pNext;
  }
  char *GetItem(){
    return m_Data.GetBuffer();
  }
};

class MyStringList {
  MyStringListItem *m_pTop;
 public:
  MyStringList(){
    m_pTop = NULL;
  }
  ~MyStringList(){
    if (m_pTop)
      delete m_pTop;
  }
  MyStringListItem *GetTop(){
    return m_pTop;
  }
  MyStringListItem *GetNext(MyStringListItem *p){
    return p->m_pNext;
  }
  void AddTail(const char *c){
    MyStringListItem *pNew = new MyStringListItem(c);
    if (!m_pTop) {
      m_pTop = pNew;
    } else {
      MyStringListItem *pTmp = m_pTop;
      while (pTmp->m_pNext)
	pTmp=pTmp->m_pNext;
      pTmp->m_pNext = pNew;
    }
  }
  void AddTop(const char *c){
    MyStringListItem *pNew = new MyStringListItem(c);
    pNew->m_pNext = m_pTop;
    m_pTop = pNew;
  }
  bool LoadFile(const char *);
  bool SaveFile(const char *);
  bool Find(const char *c){
    MyStringListItem *pTmp = m_pTop;
    while (pTmp){
      if (!strcmp(pTmp->m_Data.GetBuffer(), c))
	return true;
      pTmp=pTmp->m_pNext;
    }
    return false;
  }
  void Clear(){
    if (m_pTop)
      delete m_pTop;
    m_pTop=NULL;
  }
};

extern void Chop(char *);
extern char *SkipSpace(char *);
extern char *NextToken(char *);
extern bool StrFind(const char *, const char *, bool, bool);

#endif
