// SpamMailEraser
//   POPnetLib.c++
//       2004 Y.Senta

#include "POPnetLib.h"
#include "md5.h"

#include <stdlib.h>

#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <memory.h>

#define POP_PORT 110
#define IDFILE "/.spammaileraser_idlog"

POPnetLib::POPnetLib(){
  m_S = 0;
}

POPnetLib::~POPnetLib(){
  if (m_S)
    close(m_S);
}
char *POPnetLib::GetOneLine(){
  static char szBuffer[ONELINESIZE];
  
  for (int i=0;;){
    char c;
    int n=recv(m_S, &c, 1, 0);
    if (n<=0)
      return NULL;
    
    if (c == '\r')
      continue;
    if (c == '\n')
      c = 0;
    szBuffer[i]=c;
    if (!c)
      return szBuffer;
    i++;

    if (i==sizeof(szBuffer))
      return NULL;
  }
}

bool POPnetLib::IsOK(const char *c){
  if (!c)
    return false;
  if (!strncmp(c, "+OK", 3))
    return true;
  send(m_S, "quit\n", 5, 0);
  return false;
}

bool POPnetLib::Init(const char *szInitFile){
  if (!m_Cmd.Load(szInitFile))
    return false;
  
  // Connect
  struct hostent *hp;
  if ((hp=gethostbyname(m_Cmd.GetHost()))==NULL) {
    fprintf (stderr, "ERROR: connect error\n");
    return false;
  }
  
  m_S=socket(AF_INET,SOCK_STREAM,0);
  
  struct sockaddr_in sin;
  
  sin.sin_family = AF_INET;
  sin.sin_port=htons(POP_PORT);
  
  memcpy (&sin.sin_addr,hp->h_addr,hp->h_length);
  if (connect(m_S,(struct sockaddr *)&sin,sizeof(sin)) < 0){
    close(m_S);
    m_S=0;
    fprintf (stderr, "ERROR: socket open error\n");
    return false;
  }
  
  // login
  MyString str;
  
  char *p=GetOneLine(); // HELLO
  if (!IsOK(p))
    return false;

  bool bConnect=false;

  if (m_Cmd.IsAPOPMode()){
    p = strstr(p, "<");
    if (p){
      char buf[ONELINESIZE];
      strcpy(buf, p);
      strcat(buf, m_Cmd.GetPass());

      md5 hash;
      str = "APOP ";
      str += m_Cmd.GetID();
      str += " ";
      str += hash.GetString(buf);
      str += "\n";
      send(m_S, str.GetBuffer(), str.GetSize(), 0);
      p=GetOneLine();
      if (!p)
	return false;
      if (!strncmp(p, "+OK", 3))
	bConnect=true;
    }
    if (!bConnect)
      fprintf(stderr,"APOP connection fail. Try POP3 connection\n");
  }
  
  if (!bConnect){
    str = "USER ";
    str += m_Cmd.GetID();
    str += "\n";
    send(m_S, str.GetBuffer(), str.GetSize(), 0);
  
    p=GetOneLine();
    if (!IsOK(p))
      return false;
  
    str = "PASS ";
    str += m_Cmd.GetPass();
    str += "\n";
    send(m_S, str.GetBuffer(), str.GetSize(), 0);
  
    p=GetOneLine();
    if (!IsOK(p)){
      fprintf(stderr,"POP3 connection fail. Exit\n");
      return false;
    }
  }
  MyString strIDFile = getenv("HOME");
  strIDFile += IDFILE;
  
  m_IDlist.LoadFile(strIDFile.GetBuffer());
  
  return true;
}

bool POPnetLib::Check(bool bDelete, bool bShow){
  MyStringList OKIDList;
  MyStringList head;
  
  send(m_S, "STAT\n", 5, 0);
  char *p=GetOneLine();
  if (!IsOK(p))
    return false;
  
  int nCount, nDummy;
  if (sscanf(p, "+OK %d %d", &nCount, &nDummy)!=2)
    nCount = 0;
  
  char szBuf[FILELINESIZE];
  char szID[FILELINESIZE];
  
  for (int i=0; i<nCount; i++){
    sprintf(szBuf, "UIDL %d\n", i+1);
    send(m_S, szBuf, strlen(szBuf), 0);
    p=GetOneLine();
    if (!IsOK(p))
      return false;
    
    char *q=p+3;
    while (*q==' ')
      q++;
    while (*q!=' ')
      q++;
    while (*q==' ')
      q++;
    strcpy(szID, q);
    if (m_IDlist.Find(szID)){
      OKIDList.AddTop(szID);
      continue;
    }
    
    sprintf(szBuf, "TOP %d 0\n", i+1);
    send(m_S, szBuf, strlen(szBuf), 0);
    p=GetOneLine();
    if (!IsOK(p))
      return false;
    
    head.Clear();
    
    if (bShow)
      printf("--- Header of %s ---\n", szID);
    
    for (;;) {
      p=GetOneLine();
      if (!p)
	return false;
      if (!strcmp(p, "."))
	break;
      head.AddTail(p);
      
      if (bShow)
	printf("%s\n", p);
    }
    if (m_Cmd.DoCheck(head)){
      OKIDList.AddTop(szID);
      if (bShow)
	printf("===> OK\n");
    } else {
      if (bDelete){
	sprintf(szBuf, "DELE %d\n", i+1);
	send(m_S, szBuf, strlen(szBuf), 0);
	p=GetOneLine();
	if (!IsOK(p))
	  return false;
      }
      if (bShow)
	printf("===> NG\n");
    }
  }
  send(m_S, "quit\n", 5, 0);
  
  if (bDelete){
    MyString strIDFile = getenv("HOME");
    strIDFile += IDFILE;
  
    OKIDList.SaveFile(strIDFile.GetBuffer());
  }
  return true;
}
