//   md5.h
//       2004 Y.Senta

#include <stdio.h>
#include <string.h>
#include "md5.h"

md5::md5(){
}

md5::~md5(){
}

char *md5::GetString(const char *c){
  Init();
  Update ((unsigned char *)c, strlen(c));
  Final ();
  
  static char ret[33];
  for (int i=0;i<16;i++)
    sprintf(ret+i*2,"%02x",m_hash[i]);
  
  return ret;
}

void md5::Init(){
  m_Contex.count[0] = m_Contex.count[1] = 0;
  m_Contex.state[0] = 0x67452301;
  m_Contex.state[1] = 0xefcdab89;
  m_Contex.state[2] = 0x98badcfe;
  m_Contex.state[3] = 0x10325476;
}

void md5::Update(unsigned char *c, int len){
  // Compute number of bytes mod 64
  unsigned int index = (unsigned int)((m_Contex.count[0] >> 3) & 0x3F);
  
  // Update number of bits
  if ((m_Contex.count[0] += ((UINT4)len << 3)) < ((UINT4)len << 3))
    m_Contex.count[1]++;
  m_Contex.count[1] += ((UINT4)len >> 29);
  
  int partLen = 64 - index;
  
  // Transform as many times as possible.
  int i;
  if (len >= partLen) {
    ContexCpy (index, c, partLen);
    Transform (m_Contex.buffer);
    
    for (i=partLen; i+63 < len; i += 64)
      Transform (&c[i]);
    
    index = 0;
  } else 
    i = 0;
  // Buffer remaining input
  
  ContexCpy(index, &c[i],len-i);
}

void md5::ContexCpy(int index, const unsigned char *input, int len){
  int i;
  
  for (i = 0; i < len; i++)
    m_Contex.buffer[index+i] = input[i];
}

void md5::Decode (UINT4 *output, unsigned char *input, unsigned int len) {
  unsigned int i, j;
  
  for (i = 0, j = 0; j < len; i++, j += 4)
    output[i] = ((UINT4)input[j]) | (((UINT4)input[j+1]) << 8) |
      (((UINT4)input[j+2]) << 16) | (((UINT4)input[j+3]) << 24);
}


void md5::Transform (unsigned char block[64]){
  UINT4 a = m_Contex.state[0], b = m_Contex.state[1], c = m_Contex.state[2], d = m_Contex.state[3], x[16];
  Decode (x, block, 64);
  
  // Round 1
  FF (&a, b, c, d, x[ 0],  7, 0xd76aa478); // 1
  FF (&d, a, b, c, x[ 1], 12, 0xe8c7b756); // 2
  FF (&c, d, a, b, x[ 2], 17, 0x242070db); // 3
  FF (&b, c, d, a, x[ 3], 22, 0xc1bdceee); // 4
  FF (&a, b, c, d, x[ 4],  7, 0xf57c0faf); // 5
  FF (&d, a, b, c, x[ 5], 12, 0x4787c62a); // 6
  FF (&c, d, a, b, x[ 6], 17, 0xa8304613); // 7
  FF (&b, c, d, a, x[ 7], 22, 0xfd469501); // 8
  FF (&a, b, c, d, x[ 8],  7, 0x698098d8); // 9
  FF (&d, a, b, c, x[ 9], 12, 0x8b44f7af); // 10
  FF (&c, d, a, b, x[10], 17, 0xffff5bb1); // 11
  FF (&b, c, d, a, x[11], 22, 0x895cd7be); // 12
  FF (&a, b, c, d, x[12],  7, 0x6b901122); // 13
  FF (&d, a, b, c, x[13], 12, 0xfd987193); // 14
  FF (&c, d, a, b, x[14], 17, 0xa679438e); // 15
  FF (&b, c, d, a, x[15], 22, 0x49b40821); // 16
  
  // Round 2
  GG (&a, b, c, d, x[ 1],  5, 0xf61e2562); // 17
  GG (&d, a, b, c, x[ 6],  9, 0xc040b340); // 18
  GG (&c, d, a, b, x[11], 14, 0x265e5a51); // 19
  GG (&b, c, d, a, x[ 0], 20, 0xe9b6c7aa); // 20
  GG (&a, b, c, d, x[ 5],  5, 0xd62f105d); // 21
  GG (&d, a, b, c, x[10],  9, 0x02441453); // 22
  GG (&c, d, a, b, x[15], 14, 0xd8a1e681); // 23
  GG (&b, c, d, a, x[ 4], 20, 0xe7d3fbc8); // 24
  GG (&a, b, c, d, x[ 9],  5, 0x21e1cde6); // 25
  GG (&d, a, b, c, x[14],  9, 0xc33707d6); // 26
  GG (&c, d, a, b, x[ 3], 14, 0xf4d50d87); // 27
  GG (&b, c, d, a, x[ 8], 20, 0x455a14ed); // 28
  GG (&a, b, c, d, x[13],  5, 0xa9e3e905); // 29
  GG (&d, a, b, c, x[ 2],  9, 0xfcefa3f8); // 30
  GG (&c, d, a, b, x[ 7], 14, 0x676f02d9); // 31
  GG (&b, c, d, a, x[12], 20, 0x8d2a4c8a); // 32
  
  // Round 3
  HH (&a, b, c, d, x[ 5],  4, 0xfffa3942); // 33
  HH (&d, a, b, c, x[ 8], 11, 0x8771f681); // 34
  HH (&c, d, a, b, x[11], 16, 0x6d9d6122); // 35
  HH (&b, c, d, a, x[14], 23, 0xfde5380c); // 36
  HH (&a, b, c, d, x[ 1],  4, 0xa4beea44); // 37
  HH (&d, a, b, c, x[ 4], 11, 0x4bdecfa9); // 38
  HH (&c, d, a, b, x[ 7], 16, 0xf6bb4b60); // 39
  HH (&b, c, d, a, x[10], 23, 0xbebfbc70); // 40
  HH (&a, b, c, d, x[13],  4, 0x289b7ec6); // 41
  HH (&d, a, b, c, x[ 0], 11, 0xeaa127fa); // 42
  HH (&c, d, a, b, x[ 3], 16, 0xd4ef3085); // 43
  HH (&b, c, d, a, x[ 6], 23, 0x04881d05); // 44
  HH (&a, b, c, d, x[ 9],  4, 0xd9d4d039); // 45
  HH (&d, a, b, c, x[12], 11, 0xe6db99e5); // 46
  HH (&c, d, a, b, x[15], 16, 0x1fa27cf8); // 47
  HH (&b, c, d, a, x[ 2], 23, 0xc4ac5665); // 48
  
  // Round 4
  II (&a, b, c, d, x[ 0],  6, 0xf4292244); // 49
  II (&d, a, b, c, x[ 7], 10, 0x432aff97); // 50
  II (&c, d, a, b, x[14], 15, 0xab9423a7); // 51
  II (&b, c, d, a, x[ 5], 21, 0xfc93a039); // 52
  II (&a, b, c, d, x[12],  6, 0x655b59c3); // 53
  II (&d, a, b, c, x[ 3], 10, 0x8f0ccc92); // 54
  II (&c, d, a, b, x[10], 15, 0xffeff47d); // 55
  II (&b, c, d, a, x[ 1], 21, 0x85845dd1); // 56
  II (&a, b, c, d, x[ 8],  6, 0x6fa87e4f); // 57
  II (&d, a, b, c, x[15], 10, 0xfe2ce6e0); // 58
  II (&c, d, a, b, x[ 6], 15, 0xa3014314); // 59
  II (&b, c, d, a, x[13], 21, 0x4e0811a1); // 60
  II (&a, b, c, d, x[ 4],  6, 0xf7537e82); // 61
  II (&d, a, b, c, x[11], 10, 0xbd3af235); // 62
  II (&c, d, a, b, x[ 2], 15, 0x2ad7d2bb); // 63
  II (&b, c, d, a, x[ 9], 21, 0xeb86d391); // 64
  
  m_Contex.state[0] += a;
  m_Contex.state[1] += b;
  m_Contex.state[2] += c;
  m_Contex.state[3] += d;
  
  // Zeroize sensitive information.
  
  //	MD5_memset ((POINTER)x, 0, sizeof (x));
  
}

void md5::Final() {
  static unsigned char PADDING[64] = {
    0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
  };
  
  unsigned char bits[8];
  unsigned int index, padLen;
  
  // Save number of bits
  Encode (bits, m_Contex.count, 8);
  
  // Pad out to 56 mod 64.
  
  index = (unsigned int)((m_Contex.count[0] >> 3) & 0x3f);
  padLen = (index < 56) ? (56 - index) : (120 - index);
  Update (PADDING, padLen);
  
  // Append length (before padding)
  Update (bits, 8);
  
  // Store state in digest
  Encode (m_hash, m_Contex.state, 16);
}

void md5::Encode (unsigned char *output, UINT4 *input, unsigned int len) {
  unsigned int i, j;
  
  for (i = 0, j = 0; j < len; i++, j += 4) {
    output[j] = (unsigned char)(input[i] & 0xff);
    output[j+1] = (unsigned char)((input[i] >> 8) & 0xff);
    output[j+2] = (unsigned char)((input[i] >> 16) & 0xff);
    output[j+3] = (unsigned char)((input[i] >> 24) & 0xff);
  }
}
